#!/bin/bash
#------------------------------------------------------------------
# solaris_gos_timeout-install.sh
#
# Author:  NetApp, Inc.
# Date:    21-DEC-2009
# Release: 5.2.111.7428
# Purpose: This script will set the required I/O timeout setting and 
#          sd driver parameters on a Solaris 10 Guest OS
#          for connectivity to IBM N series storage systems.
#
# Note:    This script is required to be run by root.  
#
# Copyright (c) 2008-2009 NetApp, Inc.
# All rights reserved.
#-------------------------------------------------------------------
# Use a 190 second I/O Timeout
IO_TIMEOUT=0xbe
IO_TIMEOUT_DEC=190
REBOOT_NOW=""
OLD_TUNABLES=0
VERSION=""

#
# Set the tunables depending on the version of Solaris 10
#
#

function set_tunables { 

     # These comments are common to both updates  
     echo "# IBM N series sd driver parameters for RDM and VMFS LUNs." >> /kernel/drv/sd.conf
   
      
if [ $OLD_TUNABLES -eq 1 ]; then

    echo "# This is a bitmap representation of the set of configuration parameters for the " >> /kernel/drv/sd.conf
    echo "# sd disk driver." >> /kernel/drv/sd.conf
    echo "#    BIT  Definition" >> /kernel/drv/sd.conf
    echo "#    ===  ==========" >> /kernel/drv/sd.conf
    echo "#     0   throttle (max)" >> /kernel/drv/sd.conf
    echo "#     1   controller type" >> /kernel/drv/sd.conf
    echo "#     2   fabricate device id" >> /kernel/drv/sd.conf
    echo "#     3   disable caching" >> /kernel/drv/sd.conf
    echo "#     4   play BCD" >> /kernel/drv/sd.conf
    echo "#     5   read subchannel BCD" >> /kernel/drv/sd.conf
    echo "#     6   read TOC TRK BCD" >> /kernel/drv/sd.conf
    echo "#     7   read TOC ADDR BCD" >> /kernel/drv/sd.conf
    echo "#     8   no READ_HDR" >> /kernel/drv/sd.conf
    echo "#     9   read CD XD4" >> /kernel/drv/sd.conf
    echo "#    10   not ready retries" >> /kernel/drv/sd.conf
    echo "#    11   busy retries" >> /kernel/drv/sd.conf
    echo "#    12   reset retries" >> /kernel/drv/sd.conf
    echo "#    13   reservation release time" >> /kernel/drv/sd.conf
    echo "#    14   TUR check" >> /kernel/drv/sd.conf
    echo "#    15   throttle (min)" >> /kernel/drv/sd.conf
    echo "#    16   disable disksort" >> /kernel/drv/sd.conf
    echo "#    17   enable LUN reset" >> /kernel/drv/sd.conf
    echo "# The following parameters are modified:" >> /kernel/drv/sd.conf
    echo "#    max. throttle = 32" >> /kernel/drv/sd.conf
    echo "#    not ready retries = 300" >> /kernel/drv/sd.conf
    echo "#    busy retries = 300" >> /kernel/drv/sd.conf
    echo "#    reset retries = 30" >> /kernel/drv/sd.conf
    echo "#    min. throttle = 8" >> /kernel/drv/sd.conf  
    echo 'sd-config-list="NETAPP LUN", "ontapRDM-sd-config";' >> /kernel/drv/sd.conf
    echo 'ontapRDM-sd-config=1,0x9c01,32,0,0,0,0,0,0,0,0,0,300,300,30,0,0,8,0,0;' >> /kernel/drv/sd.conf
    echo 'sd-config-list="VMWARE LUN", "vmfs-sd-config";' >> /kernel/drv/sd.conf
    echo  'vmfs-sd-config=1,0x9c01,32,0,0,0,0,0,0,0,0,0,300,300,30,0,0,8,0,0;' >> /kernel/drv/sd.conf
    echo "#END IBM N series sd driver parameters" >> /kernel/drv/sd.conf    
else  
    echo "# retries-busy     = 300" >> /kernel/drv/sd.conf
    echo "# retries-timeout   = 16" >> /kernel/drv/sd.conf
    echo "# retries-notready = 300" >> /kernel/drv/sd.conf
    echo "# retries-reset    = 30" >> /kernel/drv/sd.conf
    echo "# throttle-max     = 32" >> /kernel/drv/sd.conf
    echo "# throttle-min     = 8" >> /kernel/drv/sd.conf
    echo "#END IBM N series sd driver parameters" >> /kernel/drv/sd.conf
    echo 'sd-config-list="NETAPP LUN", "ontapRDM-sd-config";'  >> /kernel/drv/sd.conf
    echo 'ontapRDM-sd-config="NETAPP LUN","retries-busy:300,retries-timeout:16,retries-notready:300,retries-reset:30,throttle-max:32,throttle-min:8";' >> /kernel/drv/sd.conf
    echo 'sd-config-list="VMWARE LUN", "vmfs-sd-config";' >> /kernel/drv/sd.conf
    echo 'vmfs-sd-config="VMWARE LUN","retries-busy:300,retries-timeout:16,retries-notready:300,retries-reset:30,throttle-max:32,throttle-min:8";' >> /kernel/drv/sd.conf  
    echo "#END IBM N series sd driver parameters" >> /kernel/drv/sd.conf

fi;


}


#
# MAIN PROGRAM
#
#
#

# Ensure the user is running as root
USER=`/usr/ucb/whoami`;

if [ "$USER" != "root" ]
then
   echo "";
   echo "This program must be run as root.";
   echo "Exiting.";
   echo "";
   exit 2;
fi;


# Verify this is Solaris 10
cat /etc/release | grep "Solaris 10" > /dev/null 2>&1;
rc=$?
if [[ $rc -ne 0 ]]; then
   echo "";
   echo "This script is designed for Solaris 10.";
   echo "Please run the appropriate script for your GOS type.";
   echo "Exiting.";
   echo "";
   exit 4;
fi;

# Check the update version. If it is less than update 7, use old tunables
VERSION=`cat /etc/release |grep s10x | awk -F_ '{print $2}' | sed "s/wos//" | sed "s/u//"`

if [[ ${VERSION} < 7 ]]; then
	OLD_TUNABLES=1;
fi;


# Check if timeout is set
# Set the Global I/O Timeout if it is not set already

cat /etc/system | grep sd:sd_io_time=0xbe > /dev/null 2>&1;
rc=$?;
if [ $rc -eq 0 ]; then
     echo "I/O Timeout is set to 190 seconds";
else

  echo "# I/O Timeout setting (${IO_TIMEOUT_DEC}) required for connectivity to IBM N series storage systems" >> /etc/system
  echo "set sd:sd_io_time=${IO_TIMEOUT}" >> /etc/system
  echo "Setting I/O Timeout to 190 seconds";

fi;


# If Update 6 is detected, use a set of different recommended tunables than Update 7 and higher versions of Solaris
if [ $OLD_TUNABLES -eq 1 ]; then
  
  cat /kernel/drv/sd.conf | grep "1,0x9c01,32,0,0,0,0,0,0,0,0,0,300,300,30,0,0,8,0,0" > /dev/null 2>&1;  
  rc=$?  
  if [ $rc -ne 0 ]; then
        echo "Setting I/O Tunables";
        set_tunables;
  else
     echo "I/O Tunables are set";
  fi;
else    

  # Check if tunables are already in the file
  cat /kernel/drv/sd.conf | grep "retries-busy:300,retries-timeout:16,retries-notready:300,retries-reset:30,throttle-max:32,throttle-min:8" > /dev/null 2>&1;
  rc=$? 
   if [ $rc -ne 0 ]; then
       echo "Setting I/O Tunables";
       set_tunables;
   else
      echo "I/O Tunables are set";
   fi;
fi;
   
 
    

echo ""
echo "You must reboot for the changes to take effect."
echo ""
until [ "$REBOOT_NOW" = "yes" ] || [ "$REBOOT_NOW" = "no" ]
do
   echo -n "Do you want to reboot now (yes or no)?> "
   read RESPONSE
   # convert RESPONSE to lowercase
   REBOOT_NOW=`echo $RESPONSE |  tr '[:upper:]' '[:lower:]'`
done



if [ "$REBOOT_NOW" = "yes" ]; then
   init 6
else
   echo ""
   echo "You must reboot for the changes to take effect."
   echo ""
fi;


